@@ -5,7 +5,7 @@
 //              with the application.
 // ---------------------------------------------------------------------------
 // $Source: src/lib.rs $
-// $Revision: 2022-01-16T23:54:45+01:00 $
+// $Revision: 2022-02-12T07:05:40+01:00 $
 
 #![doc = include_str ! ("../README.md")]
 #![cfg_attr(not(feature = "std"), no_std)]
@@ -17,11 +17,9 @@ use core::ops::{Add, Div, Mul, Sub};
 #[cfg(feature = "fpdec")]
 pub use fpdec::{Dec, Decimal};
 pub use si_prefixes::SIPrefix;
-pub use unitless::{Unitless, ONE};
 
 pub mod prelude;
 mod si_prefixes;
-mod unitless;
 
 #[cfg(feature = "datavolume")]
 pub mod datavolume;
@@ -99,12 +97,37 @@ macro_rules! Amnt {
 
 /// The abstract type of units used to define quantities.
 pub trait Unit: Copy + Eq + PartialEq + Sized + Mul<AmountT> {
+    /// Associated type of quantity
+    type QuantityType: Quantity<UnitType = Self>;
+
     /// Optional unit used as reference for scaling the units.
     const REF_UNIT: Option<Self>;
 
     /// Returns an iterator over the variants of `Self`.
     fn iter<'a>() -> core::slice::Iter<'a, Self>;
 
+    /// Returns `Some(unit)` where `unit.symbol()` == `symbol`, or `None` if
+    /// there is no such unit.
+    fn from_symbol(symbol: &str) -> Option<Self> {
+        for unit in Self::iter() {
+            if unit.symbol() == symbol {
+                return Some(*unit);
+            }
+        }
+        None
+    }
+
+    /// Returns `Some(unit)` where `unit.scale()` == `Some(amnt)`, or `None` if
+    /// there is no such unit.
+    fn from_scale(amnt: AmountT) -> Option<Self> {
+        for unit in Self::iter() {
+            if unit.scale() == Some(amnt) {
+                return Some(*unit);
+            }
+        }
+        None
+    }
+
     /// Returns the name of `self`.
     fn name(&self) -> &'static str;
 
@@ -127,34 +150,59 @@ pub trait Unit: Copy + Eq + PartialEq + Sized + Mul<AmountT> {
     /// Returns `Some(factor)` so that `factor` * `other` == 1 * `self`, or
     /// `None` if `Self::REF_UNIT` is `None`.
     fn ratio(&self, other: &Self) -> Option<AmountT> {
-        match (self.scale(), other.scale()) {
-            (Some(from), Some(into)) => Some(from / into),
-            _ => None,
+        match Self::REF_UNIT {
+            Some(_) => Some(self.scale().unwrap() / other.scale().unwrap()),
+            None => None,
         }
     }
 }
 
 /// The abstract type of quantities.
-pub trait Quantity<U: Unit>:
+pub trait Quantity:
     Copy + Sized + Add<Self> + Sub<Self> + Div<Self> + Mul<AmountT>
 {
-    /// Returns an iterator over the variants of `U`.
-    fn iter_units<'a>() -> core::slice::Iter<'a, U> {
-        U::iter()
+    /// Associated type of unit
+    type UnitType: Unit<QuantityType = Self>;
+
+    /// Returns an iterator over the variants of `Self::UnitType`.
+    fn iter_units<'a>() -> core::slice::Iter<'a, Self::UnitType> {
+        Self::UnitType::iter()
     }
 
-    /// Returns a new instance of `Quantity<U>`.
-    fn new(amount: AmountT, unit: U) -> Self;
+    /// Returns `Some(unit)` where `unit.symbol()` == `symbol`, or `None` if
+    /// there is no such unit.
+    fn unit_from_symbol(symbol: &str) -> Option<Self::UnitType> {
+        for unit in Self::iter_units() {
+            if unit.symbol() == symbol {
+                return Some(*unit);
+            }
+        }
+        None
+    }
+
+    /// Returns `Some(unit)` where `unit.scale()` == `Some(amnt)`, or `None` if
+    /// there is no such unit.
+    fn unit_from_scale(amnt: AmountT) -> Option<Self::UnitType> {
+        for unit in Self::iter_units() {
+            if unit.scale() == Some(amnt) {
+                return Some(*unit);
+            }
+        }
+        None
+    }
+
+    /// Returns a new instance of the type implementing `Quantity`.
+    fn new(amount: AmountT, unit: Self::UnitType) -> Self;
 
     /// Returns the amount of `self`.
     fn amount(&self) -> AmountT;
 
     /// Returns the unit of `self`.
-    fn unit(&self) -> U;
+    fn unit(&self) -> Self::UnitType;
 
     /// Returns `Some(factor)` so that `factor` * `unit` == `self`, or `None` if
     /// such a factor can't be determined.
-    fn equiv_amount(&self, unit: U) -> Option<AmountT> {
+    fn equiv_amount(&self, unit: Self::UnitType) -> Option<AmountT> {
         if self.unit() == unit {
             Some(self.amount())
         } else {
@@ -165,128 +213,154 @@ pub trait Quantity<U: Unit>:
 
     /// Returns `Some(qty)` where `qty` == `self` and `qty.unit()` is `to_unit`,
     /// or `None` if the conversion can't be done.
-    fn convert(&self, to_unit: U) -> Option<Self> {
+    fn convert(&self, to_unit: Self::UnitType) -> Option<Self> {
         Some(Self::new(self.equiv_amount(to_unit)?, to_unit))
     }
-}
 
-#[derive(Copy, Clone, Debug)]
-/// Generic struct that implements trait `Quantity`.
-pub struct Qty<U: Unit> {
-    amount: AmountT,
-    unit: U,
-}
-
-impl<U: Unit> fmt::Display for Qty<U> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match self.unit.symbol() {
-            "" => write!(f, "{}", self.amount),
-            _ => write!(f, "{} {}", self.amount, self.unit.symbol()),
+        match self.unit().symbol() {
+            "" => write!(f, "{}", self.amount()),
+            _ => write!(f, "{} {}", self.amount(), self.unit().symbol()),
+        }
+    }
+
+    fn eq(&self, other: &Self) -> bool {
+        if self.unit() == other.unit() {
+            self.amount() == other.amount()
+        } else {
+            match other.equiv_amount(self.unit()) {
+                None => false,
+                Some(equiv_amount) => self.amount() == equiv_amount,
+            }
+        }
+    }
+
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        if self.unit() == other.unit() {
+            PartialOrd::partial_cmp(&self.amount(), &other.amount())
+        } else {
+            match self.equiv_amount(other.unit()) {
+                None => None,
+                Some(equiv_amount) => {
+                    PartialOrd::partial_cmp(&equiv_amount, &other.amount())
+                }
+            }
+        }
+    }
+
+    fn add(self, rhs: Self) -> Self {
+        match rhs.equiv_amount(self.unit()) {
+            Some(equiv) => Self::new(self.amount() + equiv, self.unit()),
+            None => panic!("Incompatible units!"),
+        }
+    }
+
+    fn sub(self, rhs: Self) -> Self {
+        match rhs.equiv_amount(self.unit()) {
+            Some(equiv) => Self::new(self.amount() - equiv, self.unit()),
+            None => panic!("Incompatible units!"),
+        }
+    }
+
+    fn div(self, rhs: Self) -> AmountT {
+        match rhs.equiv_amount(self.unit()) {
+            Some(equiv) => self.amount() / equiv,
+            None => panic!("Incompatible units!"),
         }
     }
 }
 
-impl<U: Unit> Quantity<U> for Qty<U> {
+/// Trait for quantities having a reference unit
+pub trait HasRefUnit: Quantity {
+    /// Unit used as reference for scaling the units of `Self::UnitType`.
+    const REF_UNIT: <Self as Quantity>::UnitType;
+
+    #[doc(hidden)]
+    /// Returns a new instance of the type implementing `HasRefUnit`, equivalent
+    /// to `amount * Self::REF_UNIT`, converted to the unit with the greatest
+    /// scale less than or equal to `amount` or - if there is no such unit - to
+    /// the unit with the smallest scale greater than `amount`, in any case
+    /// taking only SI units into account if Self::REF_UNIT is a SI unit.
+    fn _fit(amount: AmountT) -> Self {
+        let take_all = Self::REF_UNIT.si_prefix().is_none();
+        let mut it =
+            Self::iter_units().filter(|u| take_all || u.si_prefix().is_some());
+        // `it` returns atleast the reference unit, so its safe to unwrap here
+        let first = it.next().unwrap();
+        let last = it
+            .filter(|u| u.scale().is_some() && u.scale().unwrap() <= amount)
+            .last();
+        match last {
+            Some(unit) => Self::new(amount / unit.scale().unwrap(), *unit),
+            None => Self::new(amount / first.scale().unwrap(), *first),
+        }
+    }
+}
+
+/// The "unit" of the "unitless" quantity.
+#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+pub enum One {
+    One,
+}
+
+impl One {
+    const VARIANTS: [Self; 1] = [ONE];
+}
+
+/// Special singleton used as "unit" for the "unitless" quantity.
+pub const ONE: One = One::One;
+
+impl Unit for One {
+    type QuantityType = AmountT;
+    const REF_UNIT: Option<Self> = None;
+    fn iter<'a>() -> core::slice::Iter<'a, Self> {
+        Self::VARIANTS.iter()
+    }
+    fn name(&self) -> &'static str {
+        "One"
+    }
+    fn symbol(&self) -> &'static str {
+        ""
+    }
+    fn si_prefix(&self) -> Option<SIPrefix> {
+        None
+    }
+    fn scale(&self) -> Option<AmountT> {
+        None
+    }
+}
+
+impl Mul<One> for AmountT {
+    type Output = AmountT;
     #[inline(always)]
-    fn new(amount: AmountT, unit: U) -> Self {
-        Self { amount, unit }
+    fn mul(self, _rhs: One) -> Self::Output {
+        self
+    }
+}
+
+impl Mul<AmountT> for One {
+    type Output = AmountT;
+    #[inline(always)]
+    fn mul(self, rhs: AmountT) -> Self::Output {
+        rhs
+    }
+}
+
+impl Quantity for AmountT {
+    type UnitType = One;
+
+    #[inline(always)]
+    fn new(amount: AmountT, _unit: Self::UnitType) -> Self {
+        amount
     }
 
     #[inline(always)]
     fn amount(&self) -> AmountT {
-        self.amount
+        *self
     }
 
     #[inline(always)]
-    fn unit(&self) -> U {
-        self.unit
-    }
-}
-
-impl<U: Unit> PartialEq for Qty<U> {
-    fn eq(&self, other: &Self) -> bool {
-        if self.unit == other.unit {
-            self.amount == other.amount
-        } else {
-            match other.equiv_amount(self.unit) {
-                None => false,
-                Some(equiv_amount) => self.amount == equiv_amount,
-            }
-        }
-    }
-}
-
-impl<U: Unit> Eq for Qty<U> {}
-
-impl<U: Unit> PartialOrd for Qty<U> {
-    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        if self.unit == other.unit {
-            self.amount().partial_cmp(&other.amount())
-        } else {
-            match self.equiv_amount(other.unit) {
-                None => None,
-                Some(equiv_amount) => equiv_amount.partial_cmp(&other.amount()),
-            }
-        }
-    }
-}
-
-impl<U: Unit> Add<Self> for Qty<U> {
-    type Output = Self;
-
-    fn add(self, rhs: Self) -> Self::Output {
-        match rhs.equiv_amount(self.unit) {
-            Some(equiv) => Self::Output::new(self.amount + equiv, self.unit),
-            None => panic!("Incompatible units!"),
-        }
-    }
-}
-
-impl<U: Unit> Sub<Self> for Qty<U> {
-    type Output = Self;
-
-    fn sub(self, rhs: Self) -> Self::Output {
-        match rhs.equiv_amount(self.unit) {
-            Some(equiv) => Self::Output::new(self.amount - equiv, self.unit),
-            None => panic!("Incompatible units!"),
-        }
-    }
-}
-
-impl<U: Unit> Div<Self> for Qty<U> {
-    type Output = Unitless;
-
-    fn div(self, rhs: Self) -> Self::Output {
-        match rhs.equiv_amount(self.unit) {
-            Some(equiv) => Self::Output::new(self.amount / equiv, ONE),
-            None => panic!("Incompatible units!"),
-        }
-    }
-}
-
-impl<U: Unit> Mul<Qty<U>> for AmountT {
-    type Output = Qty<U>;
-
-    #[inline(always)]
-    fn mul(self, rhs: Qty<U>) -> Self::Output {
-        Self::Output::new(self * rhs.amount, rhs.unit)
-    }
-}
-
-impl<U: Unit> Mul<AmountT> for Qty<U> {
-    type Output = Self;
-
-    #[inline(always)]
-    fn mul(self, rhs: AmountT) -> Self::Output {
-        Self::Output::new(self.amount * rhs, self.unit)
-    }
-}
-
-impl<U: Unit> Div<AmountT> for Qty<U> {
-    type Output = Self;
-
-    #[inline(always)]
-    fn div(self, rhs: AmountT) -> Self::Output {
-        Self::Output::new(self.amount / rhs, self.unit)
+    fn unit(&self) -> Self::UnitType {
+        ONE
     }
 }
