@@ -5,7 +5,7 @@
 //              with the application.
 // ---------------------------------------------------------------------------
 // $Source: tests/test_qty.rs $
-// $Revision: 2022-02-17T16:39:59+01:00 $
+// $Revision: 2022-02-17T19:35:16+01:00 $
 
 #[cfg(test)]
 mod quantity_with_ref_unit_tests {
@@ -497,7 +497,7 @@ mod derived_quantity_tests {
 
     #[quantity(Foo / Bar)]
     #[ref_unit(Qoox, "Q", "1·f/e")]
-    #[unit(Flop_per_Emil, "f/e", 1., "1·f/e")]
+    #[unit(Five_Flops_per_Emil, "ff/e", 5., "5·f/e")]
     #[unit(Milliqoox, "mQ", 0.001, "0.001·Q")]
     #[unit(Microqoox, "µQ", 0.000001, "0.000001·Q")]
     #[unit(Kiloqoox, "kQ", 1000., "1000·Q")]
